package net.dries007.tfc.objects.biomes;

import net.minecraft.world.biome.Biome;
import net.minecraftforge.event.RegistryEvent;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.registry.GameRegistry;
import net.minecraftforge.registries.IForgeRegistry;

import static net.dries007.tfc.Constants.MOD_ID;
import static net.dries007.tfc.Constants.MOD_NAME;

@SuppressWarnings("ConstantConditions")
@Mod.EventBusSubscriber(modid = MOD_ID)
@GameRegistry.ObjectHolder(MOD_ID)
public final class BiomesTFC
{
    private BiomesTFC() {}

    public static final BiomeTFC OCEAN = null;
    public static final BiomeTFC RIVER = null;
    public static final BiomeTFC HELL = null;
    public static final BiomeTFC BEACH = null;
    public static final BiomeTFC GRAVEL_BEACH = null;
    public static final BiomeTFC HIGH_HILLS = null;
    public static final BiomeTFC PLAINS = null;
    public static final BiomeTFC SWAMPLAND = null;
    public static final BiomeTFC HIGH_HILLS_EDGE = null;
    public static final BiomeTFC ROLLING_HILLS = null;
    public static final BiomeTFC MOUNTAINS = null;
    public static final BiomeTFC MOUNTAINS_EDGE = null;
    public static final BiomeTFC HIGH_PLAINS = null;
    public static final BiomeTFC DEEP_OCEAN = null;
    public static final BiomeTFC LAKE = null;

    @SubscribeEvent
    public static void registerBiomes(RegistryEvent.Register<Biome> event)
    {
        IForgeRegistry<Biome> r = event.getRegistry();
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Ocean").func_185398_c(-3.6f).func_185400_d(-2.69999f)));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " River").func_185398_c(-3.2f).func_185400_d(-3.0f)));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Beach").func_185398_c(-2.69f).func_185400_d(-2.68f)));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Gravel beach").func_185398_c(-2.69f).func_185400_d(-2.68f).func_185399_a("tfc:beach")));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " High hills").func_185398_c(-1.9000001f).func_185400_d(-1.1f)));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Plains").func_185398_c(-2.6000001f).func_185400_d(-2.54f)));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Swampland").func_185398_c(-2.8f).func_185400_d(-2.6000001f), 8, 45));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " High hills edge").func_185398_c(-2.5f).func_185400_d(-2.3f).func_185399_a("tfc:high_hills")));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Rolling hills").func_185398_c(-2.6000001f).func_185400_d(-2.3f)));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Mountains").func_185398_c(-1.9000001f).func_185400_d(-1.1f)));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Mountains edge").func_185398_c(-2.3f).func_185400_d(-1.9000001f).func_185399_a("tfc:mountains")));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " High plains").func_185398_c(-2.3f).func_185400_d(-2.27f)));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Deep ocean").func_185398_c(-4.2f).func_185400_d(-2.69999f).func_185399_a("tfc:ocean")));
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Lake").func_185398_c(-3.2f).func_185400_d(-2.6990001f).func_185399_a("tfc:ocean"), 2, 5/*todo: was 0*/));

//        register(r, new BiomeTFC(new Biome.BiomeProperties("tfc_hell").setRainDisabled().setTemperature(2.0F).setRainfall(0.0F)));
    }

    private static void register(IForgeRegistry<Biome> r, Biome item)
    {
        r.register(item.setRegistryName(MOD_ID, item.func_185359_l().replace(MOD_NAME + " ", "").replace(' ', '_').toLowerCase()));
    }

    public static boolean isOceanicBiome(int id)
    {
        return Biome.func_185362_a(OCEAN) == id || Biome.func_185362_a(DEEP_OCEAN) == id;
    }

    public static boolean isMountainBiome(int id)
    {
        return Biome.func_185362_a(MOUNTAINS) == id || Biome.func_185362_a(MOUNTAINS_EDGE) == id;
    }

    public static boolean isBeachBiome(int id)
    {
        return Biome.func_185362_a(BEACH) == id || Biome.func_185362_a(GRAVEL_BEACH) == id;
    }

    public static boolean isOceanicBiome(Biome b)
    {
        return OCEAN == b || DEEP_OCEAN == b;
    }

    public static boolean isMountainBiome(Biome b)
    {
        return MOUNTAINS == b || MOUNTAINS_EDGE == b;
    }

    public static boolean isBeachBiome(Biome b)
    {
        return BEACH == b || GRAVEL_BEACH == b;
    }

    private static BiomeTFC[] PLAYER_SPAWN_BIOMES;
    private static BiomeTFC[] OVERWORLD_GENERATE_BIOMES;

    public static BiomeTFC[] getPlayerSpawnBiomes()
    {
        if (PLAYER_SPAWN_BIOMES == null)
        {
            PLAYER_SPAWN_BIOMES = new BiomeTFC[] {
                    PLAINS,
                    ROLLING_HILLS,
                    SWAMPLAND,
                    MOUNTAINS,
                    HIGH_PLAINS,
            };
        }
        return PLAYER_SPAWN_BIOMES;
    }

    public static BiomeTFC[] getOverworldGenerateBiomes()
    {
        if (OVERWORLD_GENERATE_BIOMES == null)
        {
            OVERWORLD_GENERATE_BIOMES = new BiomeTFC[] {
                    OCEAN,
                    HIGH_HILLS,
                    PLAINS,
                    HIGH_PLAINS,
                    SWAMPLAND,
                    ROLLING_HILLS,
                    MOUNTAINS,
            };
        }
        return OVERWORLD_GENERATE_BIOMES;
    }
}
