package net.dries007.tfc.objects.te;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.objects.Metal;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.items.ItemsTFC;
import net.dries007.tfc.objects.items.wood.ItemLogTFC;
import net.dries007.tfc.util.IFireable;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.ITickable;
import net.minecraft.util.NonNullList;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraftforge.common.util.Constants;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import javax.annotation.Nullable;

import static net.dries007.tfc.Constants.MOD_ID;

public class TEPitKiln extends TileEntity implements ITickable
{
    public static final ResourceLocation ID = new ResourceLocation(MOD_ID, "pit_kiln");

    public static final int STRAW_NEEDED = 8;
    public static final int WOOD_NEEDED = 8;

    private int burnTicksToGo;
    private final NonNullList<ItemStack> logs = NonNullList.func_191197_a(WOOD_NEEDED, ItemStack.field_190927_a);
    private final NonNullList<ItemStack> straw = NonNullList.func_191197_a(STRAW_NEEDED, ItemStack.field_190927_a);
    private final NonNullList<ItemStack> items = NonNullList.func_191197_a(4, ItemStack.field_190927_a);

    @Override
    public void func_73660_a()
    {
        if (burnTicksToGo > 0)
        {
            burnTicksToGo --;
            if (func_145830_o())
            {
                BlockPos above = func_174877_v().func_177982_a(0, 1, 0);
                if (field_145850_b.func_175623_d(above))
                {
                    field_145850_b.func_175656_a(above, Blocks.field_150480_ab.func_176223_P());
                }
                else
                {
                    IBlockState stateAbove = field_145850_b.func_180495_p(above);
                    if (stateAbove.func_185904_a() != Material.field_151581_o)
                    {
                        // todo: decide what to do now.
                    }
                }
            }
            if (burnTicksToGo == 0)
            {
                straw.clear();
                logs.clear();
                for (int i = 0; i < items.size(); i++)
                {
                    ItemStack stack = items.get(i);
                    Item item = stack.func_77973_b();
                    if (!(item instanceof IFireable)) continue;
                    IFireable fireable = ((IFireable) item);
                    items.set(i, fireable.getFiringResult(stack, Metal.Tier.TIER_I));
                }
            }
        }
    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox()
    {
        return new AxisAlignedBB(func_174877_v(), func_174877_v().func_177982_a(1, 1, 1));
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound compound)
    {
        super.func_189515_b(compound);
        compound.func_74772_a("burnTicksToGo", burnTicksToGo);

        {
            NBTTagList itemsNBT = new NBTTagList();
            items.forEach(i -> itemsNBT.func_74742_a(i.func_77955_b(new NBTTagCompound())));
            compound.func_74782_a("items", itemsNBT);
        }
        {
            NBTTagList strawNBT = new NBTTagList();
            straw.forEach(i -> strawNBT.func_74742_a(i.func_77955_b(new NBTTagCompound())));
            compound.func_74782_a("straw", strawNBT);
        }
        {
            NBTTagList logsNBT = new NBTTagList();
            logs.forEach(i -> logsNBT.func_74742_a(i.func_77955_b(new NBTTagCompound())));
            compound.func_74782_a("logs", logsNBT);
        }
        return compound;
    }

    @Override
    public void func_145839_a(NBTTagCompound compound)
    {
        super.func_145839_a(compound);
        burnTicksToGo = compound.func_74762_e("burnTicksToGo");
        {
            NBTTagList itemsNBT = compound.func_150295_c("items", Constants.NBT.TAG_COMPOUND);
            for (int i = 0; i < itemsNBT.func_74745_c(); i++) items.set(i, new ItemStack(itemsNBT.func_150305_b(i)));
        }
        {
            NBTTagList strawNBT = compound.func_150295_c("straw", Constants.NBT.TAG_COMPOUND);
            for (int i = 0; i < strawNBT.func_74745_c(); i++) straw.set(i, new ItemStack(strawNBT.func_150305_b(i)));
        }
        {
            NBTTagList logsNBT = compound.func_150295_c("logs", Constants.NBT.TAG_COMPOUND);
            for (int i = 0; i < logsNBT.func_74745_c(); i++) logs.set(i, new ItemStack(logsNBT.func_150305_b(i)));
        }
    }

    @Override
    public NBTTagCompound func_189517_E_()
    {
        return func_189515_b(new NBTTagCompound());
    }

    @Nullable
    @Override
    public SPacketUpdateTileEntity func_189518_D_()
    {
        return new SPacketUpdateTileEntity(field_174879_c, 127, func_189517_E_());
    }

    @Override
    public void onDataPacket(NetworkManager net, SPacketUpdateTileEntity pkt)
    {
        func_145839_a(pkt.func_148857_g());
    }

    public boolean isLit()
    {
        return burnTicksToGo > 0;
    }

    public boolean hasFuel()
    {
        return !(logs.stream().anyMatch(ItemStack::func_190926_b) || straw.stream().anyMatch(ItemStack::func_190926_b));
    }

    public boolean addItem(ItemStack heldItem, boolean x, boolean z)
    {
        int slot = 0;
        if (x) slot += 1;
        if (z) slot += 2;
        ItemStack current = items.get(slot);
        if (!current.func_190926_b()) return false;
        items.set(slot, heldItem.func_77979_a(1));
        func_70296_d();
        IBlockState state = field_145850_b.func_180495_p(field_174879_c);
        field_145850_b.func_184138_a(field_174879_c, state, state, 3);
        return false;
    }

    @SuppressWarnings("ConstantConditions")
    public void onRightClick(EntityPlayer player, ItemStack item, boolean x, boolean z)
    {
        IBlockState state = field_145850_b.func_180495_p(field_174879_c);
        field_145850_b.func_184138_a(field_174879_c, state, state, 3);
        func_70296_d();
        int count = getStrawCount();
        if (item.func_190926_b())
        {
            if (getLogCount() > 0)
            {
                ItemStack itemStack = logs.stream().filter(i -> !i.func_190926_b()).findFirst().get();
                player.func_191521_c(itemStack.func_77979_a(1));
                return;
            }
            if (getStrawCount() > 0)
            {
                ItemStack itemStack = straw.stream().filter(i -> !i.func_190926_b()).findFirst().get();
                player.func_191521_c(itemStack.func_77979_a(1));
                return;
            }
            int slot = 0;
            if (x) slot += 1;
            if (z) slot += 2;
            ItemStack current = items.get(slot);
            if (!current.func_190926_b())
            {
                player.func_191521_c(current.func_77979_a(1));
                items.set(slot, ItemStack.field_190927_a);
            }
            if (items.stream().filter(ItemStack::func_190926_b).count() == 4)
                field_145850_b.func_175655_b(field_174879_c, true);
        }
        if (item.func_77973_b() == ItemsTFC.HAY && count < STRAW_NEEDED)
        {
            addStraw(item.func_77979_a(1));
            return;
        }
        if (item.func_77973_b() == Item.func_150898_a(BlocksTFC.THATCH) && count <= STRAW_NEEDED - 4)
        {
            item.func_190918_g(1);
            addStraw(new ItemStack(ItemsTFC.HAY));
            addStraw(new ItemStack(ItemsTFC.HAY));
            addStraw(new ItemStack(ItemsTFC.HAY));
            addStraw(new ItemStack(ItemsTFC.HAY));
            return;
        }
        if (count < STRAW_NEEDED) return;
        count = getLogCount();
        if (item.func_77973_b() instanceof ItemLogTFC && count < WOOD_NEEDED)
        {
            addLog(item.func_77979_a(1));
            return;
        }
        TerraFirmaCraft.getLog().info("Fire!");
        burnTicksToGo = 600;
    }

    private void addStraw(ItemStack stack)
    {
        TerraFirmaCraft.getLog().info("addStraw {}", stack);
        for (int i = 0; i < straw.size(); i++)
        {
            if (!straw.get(i).func_190926_b()) continue;
            straw.set(i, stack);
            return;
        }
    }

    private void addLog(ItemStack stack)
    {
        TerraFirmaCraft.getLog().info("addLog {}", stack);
        for (int i = 0; i < logs.size(); i++)
        {
            if (!logs.get(i).func_190926_b()) continue;
            logs.set(i, stack);
            return;
        }
    }

    public NonNullList<ItemStack> getItems()
    {
        return items;
    }

    public int getLogCount()
    {
        return (int) logs.stream().filter(i -> !i.func_190926_b()).count();
    }

    public int getStrawCount()
    {
        return (int) straw.stream().filter(i -> !i.func_190926_b()).count();
    }
}
