package net.dries007.tfc.util;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.nbt.CompressedStreamTools;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
/**
 * todo: change to proper vanilla schematics
 */
public class Schematic implements ISchematic
{
	protected int height;
	protected int width;
	protected int centerX;
	protected int length;
	protected int centerZ;
	protected NBTTagList te;
	protected NBTTagList entities;
	protected String path;
	protected String filename;
	protected ArrayList<SchematicBlock> blockMap;
	protected AxisAlignedBB aabb;

	public Schematic(String path, String filename)
	{
		this.path = path;
		this.filename = filename;
	}

	@Override
	public boolean Load()
	{
		NBTTagCompound tree;
		blockMap = new ArrayList<SchematicBlock>();
		try
		{
			InputStream fis = getClass().getResourceAsStream(path);
			if(fis == null)
				return false;
			tree = CompressedStreamTools.func_74796_a(fis);
			height = tree.func_74765_d("Height");
			width = tree.func_74765_d("Width");
			centerX = getCenter(width);
			length = tree.func_74765_d("Length");
			centerZ = getCenter(length);
			int[] blockArray = null;
			byte[] dataArray = null;

			if(tree.func_74764_b("Blocks"))
			{
				byte[] b = tree.func_74770_j("Blocks");
				blockArray = new int[b.length];

				for(int i = 0; i < b.length; i++)
				{
					blockArray[i] = b[i];
				}
			}
			else if(tree.func_74764_b("BlocksInt"))
			{
				blockArray = tree.func_74759_k("Blocks");
			}

			dataArray = tree.func_74770_j("Data");
			te = tree.func_150295_c("TileEntities", 10);

			for(int y = 0; y < getSizeY(); y++)
			{
				for(int z = 0; z < getSizeZ(); z++)
				{
					for(int x = 0; x < getSizeX(); x++)
					{
						int index = x + getSizeX() * (z + getSizeZ() * y);
						//TODO: Add support for Tile Entity loading
						this.blockMap.add(new SchematicBlock(blockArray[index], dataArray[index], new BlockPos(x-centerX, y, z-centerZ)));
					}
				}
			}

			aabb = new AxisAlignedBB(0, 0, 0, width, height, length);
		}
		catch (FileNotFoundException e)
		{
			System.out.println("TFC FileNotFound: " + path); 
			return false;
		}
		catch (IOException e)
		{
			System.out.println("TFC IOException: " + path); 
			return false;
		}
		return true;
	}

	@Override
	public void PostProcess()
	{

	}

	@Override
	public int getSizeY()
	{
		return height;
	}

	@Override
	public int getSizeX()
	{
		return width;
	}

	@Override
	public int getSizeZ()
	{
		return length;
	}

	@Override
	public NBTTagList getTileEntities()
	{
		return te;
	}

	@Override
	public String getPath()
	{
		return path;
	}

	@Override
	public String getFileName()
	{
		return filename;
	}

	@Override
	public void setPath(String str)
	{
		path = str;
	}

	@Override
	public void setSizeY(int y)
	{
		height = y;
	}

	@Override
	public void setSizeX(int x)
	{
		width = x;
	}

	@Override
	public void setSizeZ(int z)
	{
		length = z;
	}

	@Override
	public void setTileEntities(NBTTagList tag)
	{
		te = tag;
	}

	@Override
	public NBTTagList getEntities()
	{
		return entities;
	}

	@Override
	public void setEntities(NBTTagList e)
	{
		entities = e;
	}

	@Override
	public int getCenterX()
	{
		return centerX;
	}

	@Override
	public int getCenterZ()
	{
		return centerZ;
	}

	@Override
	public ArrayList<SchematicBlock> getBlockMap() {
		return blockMap;
	}

	public AxisAlignedBB getBoundingBox(BlockPos pos)
	{
		return aabb.func_186670_a(pos);
	}

	//*****************
	// Private methods
	//*****************
	private int getCenter(int v)
	{
		if(v % 2 == 1)
			return (v + 1) / 2;
		return v / 2;
	}


	public class SchematicBlock
	{
		public IBlockState state;
		public BlockPos blockPos;
		public TileEntity tileEntity;

		public SchematicBlock(int id, int meta, BlockPos blockPos)
		{
			state = Block.func_149729_e(id).func_176203_a(meta);
			this.blockPos = blockPos;
			te = null;
		}

		public SchematicBlock(IBlockState state, BlockPos blockPos)
		{
			this.state = state;
			this.blockPos = blockPos;
			te = null;
		}

		public SchematicBlock(int id, int meta, BlockPos blockPos, TileEntity tileEntity)
		{
			state = Block.func_149729_e(id).func_176203_a(meta);
			this.blockPos = blockPos;
			this.tileEntity = tileEntity;
		}
	}
}
