package net.dries007.tfc.world.classic.worldgen;

import net.dries007.tfc.objects.Rock;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.world.classic.ChunkGenTFC;
import net.dries007.tfc.world.classic.ClimateTFC;
import net.dries007.tfc.world.classic.WorldTypeTFC;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataTFC;
import net.minecraft.block.BlockColored;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.item.EnumDyeColor;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.chunk.IChunkProvider;
import net.minecraft.world.gen.IChunkGenerator;
import net.minecraftforge.fml.common.IWorldGenerator;

import java.util.Random;

public class WorldGenSoilPits implements IWorldGenerator
{
    @Override
    public void generate(Random random, int chunkX, int chunkZ, World world, IChunkGenerator chunkGenerator, IChunkProvider chunkProvider)
    {
        if (!(chunkGenerator instanceof ChunkGenTFC)) return;
        final BlockPos chunkBlockPos = new BlockPos(chunkX << 4, 0, chunkZ << 4);

        BlockPos pos = world.func_175672_r(chunkBlockPos.func_177982_a(8 + random.nextInt(16), 0, 8 + random.nextInt(16)));
        generateClay(world, random, pos);

        pos = world.func_175672_r(chunkBlockPos.func_177982_a(8 + random.nextInt(16), 0, 8 + random.nextInt(16)));
        if (generatePeat(world, random, pos))
        {
            if (random.nextInt(5) == 0)
            {
//                if (!cloudberryGen.generate(world, random, pos)) //todo add berry gen
//                    cranberryGen.generate(world, random, pos);
            }
        }
    }

    private void generateClay(World world, Random rng, BlockPos start)
    {
        int radius = rng.nextInt(14) + 2;
        int depth = rng.nextInt(3) + 1;
        if (rng.nextInt(30) != 0 || start.func_177956_o() > WorldTypeTFC.SEALEVEL + 6) return;

        for (int x = -radius; x <= radius; x++)
        {
            for (int z = -radius; z <= radius; z++)
            {
                if (x * x + z * z > radius * radius) continue;
                final BlockPos posHorizontal = start.func_177982_a(x, 0, z);
                if (ChunkDataTFC.getRainfall(world, posHorizontal) < 500) continue;

                boolean flag = false;
                for (int y = -depth; y <= +depth; y++)
                {
                    final BlockPos pos = posHorizontal.func_177982_a(0, y, 0);
                    final IBlockState current = world.func_180495_p(pos);
                    if (BlocksTFC.isDirt(current))
                    {
                        world.func_180501_a(pos, ChunkDataTFC.getRockHeight(world, pos).getVariant(Rock.Type.CLAY).func_176223_P(), 2);
                        flag = true;
                    }
                    else if (BlocksTFC.isGrass(current))
                    {
                        world.func_180501_a(pos, ChunkDataTFC.getRockHeight(world, pos).getVariant(Rock.Type.CLAY_GRASS).func_176223_P(), 2);
                        flag = true;
                    }
                }
                if (flag && rng.nextInt(15) == 0)
                {
                    final BlockPos pos = world.func_175672_r(posHorizontal);
                    if (world.func_175623_d(pos) && BlocksTFC.isSoil(world.func_180495_p(pos.func_177982_a(0, -1, 0))))
                        world.func_180501_a(pos, Blocks.field_150325_L.func_176223_P().func_177226_a(BlockColored.field_176581_a, EnumDyeColor.YELLOW), 2); //todo: replace with plant
                }
            }
        }
    }

    private boolean generatePeat(World world, Random rng, BlockPos start)
    {
        int radius = rng.nextInt(16) + 8;
        byte depth = 2;
        boolean flag = false;

        if (rng.nextInt(30) != 0 || start.func_177956_o() > WorldTypeTFC.SEALEVEL) return false;

        for (int x = -radius; x <= radius; ++x)
        {
            for (int z = -radius; z <= radius; ++z)
            {
                if (x * x + z * z > radius * radius) continue;

                for (int y = -depth; y <= depth; ++y)
                {
                    final BlockPos pos = start.func_177982_a(x, y, z);
                    if (!ClimateTFC.isSwamp(world, pos)) continue;
                    final IBlockState current = world.func_180495_p(pos);

                    if (BlocksTFC.isGrass(current))
                    {
                        world.func_180501_a(pos, BlocksTFC.PEAT_GRASS.func_176223_P(), 2);
                        flag = true;
                    }
                    else if (BlocksTFC.isDirt(current) || BlocksTFC.isClay(current))
                    {
                        world.func_180501_a(pos, BlocksTFC.PEAT.func_176223_P(), 2);
                        flag = true;
                    }
                }
            }
        }
        return flag;
    }
}
