package net.dries007.tfc.world.classic.worldgen;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.objects.Wood;
import net.dries007.tfc.objects.blocks.wood.BlockLeavesTFC;
import net.dries007.tfc.objects.blocks.wood.BlockLogTFC;
import net.dries007.tfc.objects.blocks.wood.BlockSaplingTFC;
import net.dries007.tfc.objects.trees.TreeRegistry;
import net.dries007.tfc.objects.trees.TreeSchematicManager;
import net.dries007.tfc.util.Schematic;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.gen.feature.WorldGenAbstractTree;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;

import static net.dries007.tfc.objects.blocks.BlocksTFC.isSoil;

/**
 * Ported and modified version of TFC2 code by Bioxx
 */
public class WorldGenTree extends WorldGenAbstractTree {

    protected IBlockState saplingBlock;
    protected int generateFlag;
    protected ArrayList<IBlockState> validGroundBlocks;
    protected Wood wood;

    //protected config things
    private boolean allowBarkCoveredLogs;

    public WorldGenTree() { this(Wood.ASH,false); }

    public WorldGenTree(Wood wood, boolean notify) {

        super(notify);

        this.wood = wood;
        this.saplingBlock = BlockSaplingTFC.get(wood).func_176223_P();
        this.generateFlag = 2;

        // Each tree sub-class is responsible for using (or not using) this list as part of its generation logic.
        this.validGroundBlocks = new ArrayList<>(Arrays.asList());
        this.allowBarkCoveredLogs = true;
    }

    @Override
    public boolean func_180709_b(World world, Random rand, BlockPos pos) {

        // use grow here
        return false;
    }

    @Override
    public boolean isReplaceable(World world, BlockPos pos) {
        IBlockState blockState = world.func_180495_p(pos);
        return blockState.func_185904_a().func_76222_j() || super.func_150523_a(blockState.func_177230_c());
    }

    public void grow(World world, Random random, BlockPos blockPos, IBlockState blockState)
    {
        TerraFirmaCraft.getLog().info("Sapling.grow called");
        TreeSchematicManager treeSchematicManager = TreeRegistry.instance.managerFromString(wood.name());

        int size = random.nextInt(100 ) < 20 ? 2 : random.nextInt(100 ) < 50 ? 1: 0;

        int rotation = random.nextInt(4);

        for(int i = size; i >= 0; i--)
        {
            TreeSchematicManager.TreeSchematic randomSchematic = treeSchematicManager.getRandomSchematic(random);
            int invalidCount = 0;
            int baseValidCount = 0;
            BlockPos scanPos;

            //validate the tree area
            for(Schematic.SchematicBlock schematicBlock : randomSchematic.getBlockMap())
            {
                scanPos = rotatePos(blockPos, schematicBlock.blockPos, rotation);
                if(schematicBlock.state.func_177230_c().func_149688_o(schematicBlock.state) == Material.field_151575_d)
                {
                    if(!world.func_180495_p(scanPos).func_177230_c().func_176200_f(world, scanPos))
                        invalidCount++;

                    if(schematicBlock.blockPos.func_177956_o() == 0)
                        if(isSoil(world.func_180495_p(scanPos.func_177977_b())))
                            baseValidCount++;

                }
            }

            if(invalidCount > randomSchematic.getLogCount() / 10 || baseValidCount < randomSchematic.getBaseCount()*0.75)
                continue;


            for(Schematic.SchematicBlock schematicBlock : randomSchematic.getBlockMap())
            {
                Process(world, wood, rotatePos(blockPos, schematicBlock.blockPos, rotation), schematicBlock.state);
            }
            break;
        }
    }

    private BlockPos rotatePos(BlockPos treePos, BlockPos localPos, int rot)
    {
        int localX = treePos.func_177958_n() + (localPos.func_177958_n() * -1) - 2;
        int localZ = treePos.func_177952_p() + (localPos.func_177952_p() * -1) - 2;
        int localY = treePos.func_177956_o() + localPos.func_177956_o();

        if(rot == 0)
        {
            localX = treePos.func_177958_n() + localPos.func_177958_n() + 1;
            localZ = treePos.func_177952_p() + localPos.func_177952_p() + 1;
        }
        else if(rot == 1)
        {
            localX = treePos.func_177958_n() + localPos.func_177952_p();
            localZ = treePos.func_177952_p() + (localPos.func_177958_n() * -1) - 2;
        }
        else if(rot == 2)
        {
            localX = treePos.func_177958_n()  + (localPos.func_177952_p() * -1) -2;
            localZ = treePos.func_177952_p() + localPos.func_177958_n();
        }

        return new BlockPos(localX, localY, localZ);
    }

    private void Process(World world, Wood wood, BlockPos blockPos, IBlockState state)
    {
        BlockLogTFC block = BlockLogTFC.get(wood);
        BlockLeavesTFC leaves = BlockLeavesTFC.get(wood);

        if(state.func_177230_c().func_149688_o(state) == Material.field_151575_d)
        {
            world.func_180501_a(blockPos, block.func_176223_P(), 2);
        }
        else if(state.func_177230_c().func_149688_o(state) == Material.field_151584_j)
        {
            if(world.func_180495_p(blockPos).func_177230_c().func_176200_f(world, blockPos))
            {
                world.func_180501_a(blockPos, leaves.func_176223_P(), 2);
            }
        }
        else
        {
            world.func_175656_a(blockPos, state);
        }
    }
}
